// $Id: ResultSet.java,v 1.3 2002/11/29 11:19:03 mike Exp $

package org.z3950.zoom;
import java.util.Hashtable;
import java.util.Vector;
//import com.k_int.IR.*;
import com.k_int.IR.IRQuery;
import com.k_int.IR.SearchTask;
import com.k_int.IR.InformationFragmentSource;
import com.k_int.IR.TimeoutExceededException;
import com.k_int.IR.SearchException;
import com.k_int.IR.InformationFragment;
import com.k_int.IR.PresentException;
import com.k_int.IR.RecordFormatSpecification;

/**
 * Represents a result set held on a Z39.50 Server, as described in
 * the ZOOM Abstract API.
 * <P>
 * <B>Synopsis:</B>
 * <P>
 * <PRE>
 *	ResultSet rs = conn.search(query);
 *	int size = rs.size();
 *	for (int i = 0; i < size; i++) {
 *	    System.out.println(rs.getRecord(i).render());
 *	}
 * </PRE>
 * <P>
 * ResultSets are created only by the Connection class's
 * <CODE>search()</CODE> method, so there is no public constructor for
 * this class.
 *
 * @version	$Id: ResultSet.java,v 1.3 2002/11/29 11:19:03 mike Exp $
 * @see		Connection#search(Query)
 * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.4"
 *		       >zoom.z3950.org/api/zoom-1.1.html#3.4</A>
 */
public class ResultSet {
    Hashtable options = new Hashtable();
    private Connection conn;
    private Query q;
    private SearchTask st;
    private InformationFragmentSource ifs;

    /**
     * Returns the value of the specified option in this ResultSet
     * object, if it is defined there; otherwise returns the specified
     * option's value in the Connection object by which this ResultSet
     * was created.
     *
     * @param key	the name of the requested option
     * @return		the value of the specified option, or null if
     *			no such option has been defined.
     * @see		Connection#option(String)
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.4.2"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.4.2</A>
     */
    public String option(String key) {
	String res = (String) options.get(key);
	if (res == null) res = conn.option(key);
	return res;
    }

    /**
     * Sets the specified option in this ResultSet object to the
     * specified value, returning its previous value (as would have
     * been returned by the single-parameter version.)
     *
     * @param key	the name of the option to set
     * @param value	the value to which to set the specified option
     * @return		the old value of the specified option, if any
     * @see		#option(String)
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.4.2"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.4.2</A>
     */
    public String option(String key, String val) {
	String old = option(key);
	options.put(key, val);
	return old;
    }

    // package visibility: use only by Connection.search
    ResultSet(Connection conn, Query q) {
	this.conn = conn;
	this.q = q;
	// Nothing to actually do here: _search() does the work
    }

    // package visibility: used only by Connection.search
    //      ###	We should probably catch JZkit exceptions and map them
    //		to ZOOM ones, but that raises (har!) issues of its
    //		own: we don't want to introduce new kinds of exception
    //		such as TimeoutExceededException to the ZOOM spec
    //		merely because the substrate for this particular
    //		implementation happens to have such a thing.  Not sure
    //		what the right thing to do is here.  One option is to
    //		map each ZJkit exception in to Bib-1 diagnostic
    //		(e.g. TimeoutExceededException => 31, "Resources
    //		exhausted - no results available"), but that feels
    //		perverse.  Guidance, please!
    //
    void _search() throws TimeoutExceededException, SearchException,
    Bib1Exception {
	IRQuery e = q._JZQuery();

	String dbname = conn.option("databaseName");
	if (dbname == null) {
	    // 23 -> Specified combination of databases not supported
	    throw new Bib1Exception(23, "no database specified");
	}

	Vector dbnames = new Vector();
	dbnames.add(dbname);
	//  ###	I'm not ecstatic about this modal messing with `e'.
	//  Maybe we should make a clone of `e' and mess with that
	//  instead?  Doesn't really matter in the case, since we'll
	//  set databaseName each time through.
	e.collections = dbnames;

	// ####	This is the wrong time to specify record syntax, but
	//	I'm not sure that JZkit allows us to do it at the
	//	right time.
	String recsyn = conn.option("preferredRecordSyntax");
	if (recsyn != null) {
	    e.hints = new Hashtable();
	    e.hints.put("record_syntax", recsyn);
	    // ### The ZOOM AAPI should specify what the legal values are
	}

	//  ###	There's no way in JZkit to specify elementsetname.
	//	It's initialised to "f" in Z3950Origin, and because
	//	it's  private, there's no way to change it.
	st = conn._JZConn().createTask(e, null);
	st.evaluate(60000);
	ifs = st.getTaskResultSet();
    }

    /**
     * Returns the number of records found by the server satisfying
     * the query with which this ResultSet was created.
     *
     * @return		the number of records in the ResultSet
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.4.3"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.4.3</A>
     */
    public int size() {
	return ifs.getFragmentCount();
    }

    /**
     * Fetches the <CODE>i</CODE>th record of this ResultSet from the
     * server on the other end of the Connection with which the
     * ResultSet was created.  Counting starts from zero, so that the
     * highest acceptable value of <CODE>i</CODE> is one less than the
     * return value of the <CODE>size()</CODE> method.
     *
     * @param i		zero-based index of the record to fetch.
     * @return		a newly created Record object representing the
     *			data held on the server.
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.4.4"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.4.4</A>
     */
    public Record getRecord(int i) {
	InformationFragment frag[] = null; // assign to avoid warning
	try {
	    frag = ifs.getFragment(i+1, 1, new RecordFormatSpecification
		// ### Don't hardwire: record-syntax, schema, element-set
		("xml", null, "f"));
	} catch (PresentException ex) {
	    System.err.println(ex);
	    System.exit(1);
	}
	return new Record(frag[0]);
    }

    /**
     * Deletes this ResultSet, informing the server that it may
     * release any resources it has allocated to it.
     *
     * @see	<A href="http://zoom.z3950.org/api/zoom-1.1.html#3.4.5"
     *		               >zoom.z3950.org/api/zoom-1.1.html#3.4.5</A>
     */
    public void delete() {
	// #### go on then!
    }
}
